import IAjaxCommand = drupal.Core.IAjaxCommand;
import IAjax = drupal.Core.IAjax;
import Location from "./map/Location";
import SimpleMap from "./map/modules/simple/SimpleMap";
import LocationMap from "./map/modules/locations/LocationMap";
import BaseMap from "./map/providers/BaseMap";
import MapPoint from "./map/MapPoint";
import MapIcon from "./map/MapIcon";
import Subscriber from "./map/Subscriber";

export type BaseMapInterface = BaseMap;

export type SimpleMapInterface = SimpleMap;

interface BaseMapAbstractInterface {
  providerIsReady(): boolean
  subscribe(callback: Function): void
  init(context?: Document|DocumentFragment|Element): void
  loadScriptAsync(src: string): Promise<boolean>
}

export type LocationInterface = Location;

export type LocationMapInterface = LocationMap;

export type MapPointInterface = MapPoint;

export type MapIconInterface = MapIcon;

export type SubscriberInterface = Subscriber;

export type SubscriberCallback = (namespace: string, element: HTMLElement, options: DrupalMapOptions, mapSettings: BaseMapsSettings) => boolean|null;

export interface ExtendedMarker {
  a12sLocation?: LocationInterface
}

/**
 * The data related to a given location.
 */
export interface LocationData {

  id: string;

  title?: string;

  coordinates: string;

  address?: string;

  city?: string;

  postalCode?: string;

  [key: string]: any;

}

export interface LocationMarkerOptions {

  dataAttribute?: string

  markerOptions?: DrupalMapMarkerOptions

}

export interface LatLng {

  /**
   * Latitude in degrees.
   */
  lat: number;

  /**
   * Longitude in degrees.
   */
  lng: number;

}

export interface DrupalMapEvent {

  type: string

}

export interface DrupalMapLocationEvent extends DrupalMapEvent {

  location: Location

  locationIsActive: boolean

}

export interface DrupalMapOptionsSelectors {

  [key: string]: string

  map?: string

  locationList?: string

  locationElement?: string

}

export interface DrupalMapBaseMapSettings {

  infoWindowOptions?: any

  center?: {
    lat: number
    lng: number
  }

  initialZoom?: number

  style?: any

  clusterOptions?: any

}

export interface DrupalMapMarkerOptions {

  iconCallback?: (event: DrupalMapLocationEvent) => void

}

export interface DrupalMapMarkerInfo {

  lat: number

  lng: number

  [key: string]: any

}

export interface DrupalMapOptions {

  selectors: DrupalMapOptionsSelectors

  location?: LocationMarkerOptions

  map?: DrupalMapBaseMapSettings

  markers?: DrupalMapMarkerInfo[]

}

export interface DrupalMapInterface {

  Subscriber: typeof Subscriber

  Map: typeof BaseMap

  MapPoint?: typeof MapPoint

  Location?: typeof Location

  MapIcon?: typeof MapIcon

  instances?: Map<HTMLElement, SimpleMapInterface>

  providers?: {
    [key: string]: {
      Map: BaseMapAbstractInterface & {
        new(element: HTMLElement, settings: DrupalMapBaseMapSettings): BaseMapInterface
      }

      MarkerClusterer?: new(...args: any[]) => any
    }
  }

  modules?: {

    [key: string]: typeof SimpleMap

    SimpleMap?: typeof SimpleMap

    LocationMap?: typeof LocationMap
  }

}

export interface BaseMapsSettings {

  selector: string

  provider: string

  options?: Partial<DrupalMapOptions>

  initialized?: boolean

}

export interface SimpleMapsSettings extends BaseMapsSettings {

  markers?: string[]

}

export interface LocationMapsSettings extends BaseMapsSettings {

  enabledLocations?: string[]

}

declare namespace drupalMap {

  namespace Icon {

    export interface Size {
      height: number
      width: number
    }

    export interface Position {
      x: number
      y: number
    }

    export interface Options {

      size?: drupalMap.Icon.Size

      anchor?: drupalMap.Icon.Position

      origin?: drupalMap.Icon.Position

    }

  }

}

export interface a12sLocationsMapUpdateCommand extends IAjaxCommand {
  (
    ajax: IAjax,
    response: a12sLocationsMapUpdateResponse,
    status: number
  ): void;
}

interface a12sLocationsMapUpdateResponse {
  data: any
  enabledLocations: string[]
  selector: string
}

declare global {

  namespace drupal {

      namespace Core {

        export interface IBehaviors {

          a12sLocationsBaiduMap?: IBehavior & {
            initialized: boolean
          };

        }
    }

    interface IDrupalSettings {

      a12sLocations?: {

        locationMaps?: {
          [key: string]: LocationMapsSettings
        }

        simpleMaps?: {
          [key: string]: SimpleMapsSettings
        }

        baiduMapKey?: string

        [key: string]: string | {
          [key: string]: BaseMapsSettings
        }
      };

    }

    namespace Core {

      interface IAjaxCommands {

        a12sLocationsMapUpdate: a12sLocationsMapUpdateCommand;

      }

    }

  }

  interface Window {

    DrupalMap: DrupalMapInterface

    BMAP_PROTOCOL?: string;

    BMap_loadScriptTime?: number;

  }

}
