<?php

declare(strict_types=1);

namespace Drupal\a12s_locations\Plugin\A12sLocationsMapProvider;

use Drupal\a12s_locations\Plugin\MapProviderPluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Plugin implementation of the a12s_locations_map_provider.
 *
 * @A12sLocationsMapProvider(
 *   id = "google",
 *   label = @Translation("Google map"),
 *   description = @Translation("Integrates with Google map API.")
 * )
 */
class Google extends MapProviderPluginBase {

  public function getLibraries(): array {
    return [
      'a12s_locations/google-map',
      'a12s_locations/google-map-marker-clusterer',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function defaultValues(): array {
    return [
      'type' => 'ROADMAP',
      'style' => '',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function buildMapOptions(): array {
    $options = [];

    foreach (['type'/*, 'style'*/] as $key) {
      if (!empty($this->configuration[$key])) {
        $options[$key] = $this->configuration[$key];
      }
    }

    if (!empty($this->configuration['style'])) {
      $options['style'] = json_decode($this->configuration['style']);
    }

    // @todo handle map type in the JS part.

    return $options;
  }

  /**
   * {@inheritDoc}
   */
  public function settingsForm(array $form, FormStateInterface $formState, array $settings = []): array {
    $settings = array_replace_recursive($this->defaultValues(), $settings);

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Map type'),
      '#options' => [
        'ROADMAP' => $this->t('Roadmap'),
        'SATELLITE' => $this->t('Satellite'),
        'HYBRID' => $this->t('Hybrid'),
        'TERRAIN' => $this->t('Terrain'),
      ],
      '#default_value' => NestedArray::getValue($settings, ['type']),
      '#required' => TRUE,
    ];

    $form['style'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Map style'),
      '#description' => $this->t('The map style for Google Map uses JSON format. You can use the @url to build custom styles.', [
        '@url' => Link::fromTextAndUrl($this->t('Snazzy maps'), Url::fromUri('http://snazzymaps.com/'))->toString(),
      ]),
      '#default_value' => NestedArray::getValue($settings, ['style']),
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function validateSettingsForm(array $form, FormStateInterface $formState): void {
    if ($style = trim($formState->getValue('style', ''))) {
      // @todo use json_validate() when PHP ≥ 8.3 usage is more important.
      json_decode($style);

      if (json_last_error() !== JSON_ERROR_NONE) {
        $formState->setError($form['style'], $this->t('The provided style is not a valid JSON string.'));
      }
    }
  }

}
